/* Sample code Lissajous by Rudy Rucker, CS Dept, SJSU, Sept 1, 2003.
This code is public domain and can be freely reused. */

#include <windows.h>   // use as needed for your system
#include <gl/Gl.h>
#include <gl/glut.h>
#include <time.h> //For the clock() function used by sleep() for animation.
#include <math.h> //For cos and sine functions

GLdouble WINDOW_CX = 640.0;
GLdouble WINDOW_CY = 480.0;
GLdouble xradius = 4.0;
GLdouble yradius = 3.0;
GLdouble bobradius = 0.2;
GLdouble TIMEPERFRAME = 0.02; //Time in seconds per frame
BOOL animateflag = TRUE;
GLdouble xfreq = 5.0;
GLdouble yfreq = 3.0;
GLdouble startt = 0.0;
GLdouble dt = 0.02;
int tsteps = 80;
//<<<<<<<<<<<<<<<<<<<<<<< myInit >>>>>>>>>>>>>>>>>>>>
void myInit(void)
{
/* We call myInit once at startup. */
    glClearColor(1.0,1.0,1.0,0.0);       // set white background color
 //   glClearColor(0.0,0.0,0.0,0.0);       // set white background color
    glColor3f(0.0f, 0.0f, 0.0f);          // set the drawing color 
 	glPointSize(4.0);		       // a dot is 4 by 4 pixels
	glMatrixMode(GL_PROJECTION); 
	glLoadIdentity();
//#define PERSPECTIVE
#ifndef PERSPECTIVE
	glOrtho(-xradius, xradius, -yradius, yradius, -1, 1);
		//Arguments: left, right, bottom, top, near, far
#else //PERSPECTIVE
	gluPerspective(60.0, xradius/yradius, 0.1, 20.0);
		//Arguments: view angle in degrees, width-to-height ratio, near, far.
	gluLookAt(0.0, 0.0, 2.0 * xradius, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
		//Arguments: 3 eye coords, 3 center coords, 3 up direction components.
#endif //PERSPECTIVE
	glShadeModel(GL_FLAT);
	glEnable(GL_DEPTH_TEST);
}
//<<<<<<<<<<<<<<<<<<<<<<<< utilities >>>>>>>>>>>>>>>>>
int random(int m)
{
	return rand()%m; /* The C rand function returns a random
		int between 0 and BIGNUM (about 2 billion when using 32 bit int).
		We use the % modulo method to get a number less than m so
		that our random(m) returns a number from 0 through m-1. */
}

GLdouble randomdouble()
{
/* returns a number between 0.0 and 1.0 */
	return GLdouble(random(1000))/1000.0;
}

GLdouble random(GLdouble f)
{
/* returns a number between 0.0 and f */
	return randomdouble()*f;
}

void sleep(double secondstowait)
{
/* This makes the program spin in a loop for an interval of secondstowait.
It can make functionality a bit awkward for long secondstowait, as you aren't
polling the keyboard and mouse while sleep spins, which can make the
program feel dead.*/
	long waitticks = secondstowait * CLOCKS_PER_SEC; 
		//CLOCKS_PER_SEC is typically 30, it's a system thing.
	long goal = waitticks + clock();
	while( goal > clock() )
      ; //Spin a few thousand times.
}
//<<<<<<<<<<<<<<<<<<<<<<<< drawSphere >>>>>>>>>>>>>>>>>
void drawSphere(GLdouble x, GLdouble y, GLdouble z, GLdouble radius)
{
	glMatrixMode(GL_MODELVIEW); //Ready to move objects into place
	glPushMatrix();
	glTranslated(x, y, z);
	glutSolidSphere(radius, 16, 16); //Second two argumetns are slices and stacks.
	glPopMatrix();
}

//<<<<<<<<<<<<<<<<<<<<<<< drawFlurry >>>>>>>>>>>>>>>>>
void drawLiss() //Uses the global afreq, bfreq, startt, dt, and tsteps params.
{
	double t = startt;
	glColor3f(0.0, 0.0, 0.0);
	glBegin(GL_LINE_STRIP);
		for (int i = 0; i < tsteps; i++) 
		{
			glVertex3d(xradius * cos(xfreq * t), yradius * sin(yfreq * t), 0.0);
			t += dt;
		}
	glEnd();
	//Back up to the last vertex and draw a ball there.
	t-=dt;
	glColor3f(1.0, 1.0, 0.0);
	drawSphere(xradius * cos(xfreq * t), yradius * sin(yfreq * t), 0.0, bobradius);
}  
//<<<<<<<<<<<<<<<<<<<<<<<< myDisplay >>>>>>>>>>>>>>>>>
void myDisplay(void)
{
/* myDisplay is called once at start up and is called again whenever
the graphics window is first covered and then uncovered, making a
redraw necessary. */
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);     // clear the screen 
	//Draw window frame.
	glColor3f(1.0, 0.0, 0.0);
	glBegin(GL_LINE_LOOP);
		glVertex3d(-xradius, -yradius, 0.0);
		glVertex3d(xradius, -yradius, 0.0);
		glVertex3d(xradius, yradius, 0.0);
		glVertex3d(-xradius, yradius, 0.0);
	glEnd();
	//Draw Lissjous curve
	drawLiss();//Uses the global afreq, bfreq, startt, dt, and tsteps params.
	startt += dt; //Scoot the starting point forward.
//	glFlush()				// Single buffer: send all output to display 
	glutSwapBuffers();		//Doule buffer: flip.
}
//<<<<<<<<<<<<<<<<<<<<<<<< myAnimate >>>>>>>>>>>>>>>>>
void myAnimate(void)
{
/* myAnimate is called over and over whenever glut is idle (not busy
processing a key or mouse message). To keep this from happening too
often we put in a delay, that is, we use a spin loop to make myAnimate
take at least TIMEPERFRAME seconds. */
	if (!animateflag)
		return;
	long waitticks = TIMEPERFRAME * CLOCKS_PER_SEC; 
		//CLOCKS_PER_SEC is typically 30 "ticks" per second, as I recall.
	long goal = waitticks + clock();
		//clock() returns a long integer that counts something like "ticks".
	myDisplay();
	while( goal > clock() )
      ; //Spin if there's unused time.
}
//<<<<<<<<<<<<<<<<<<<<<<<< myKeyboard >>>>>>>>>>>>>>>>>
void myKeyboard(unsigned char key, int mx, int my)
{
/* key is the ASCII code of the key, such as 'a', 'A', 'b', 'B', and
so on.  Note that case matters.  mx and my are the current mouse
position. */
	switch(key)
	{
		case ' ': //Space
			myDisplay(); //redraw 
			break;
		case 'a':
			animateflag ^= TRUE;
			break;
		case 'q':
			exit(-1); //Terminate
			break;
	}
}
//<<<<<<<<<<<<<<<<<<<<<<<< myMouse >>>>>>>>>>>>>>>>>
void myMouse(int button, int state, int mx, int my)
{
/* Gets called when a button is clicked.  The arguments are:
button is GLUT_LEFT_BUTTON or GLUT_RIGHT_BUTTON,
state is GLUT_DOWN or GLUT_UP, 
and mx and my are window pixel coordinates, origin at window upper left. */
}
//<<<<<<<<<<<<<<<<<<<<<<<< myKeyboard >>>>>>>>>>>>>>>>>
void myMovedMouse(int mx, int my)
{
/*Gets called whenever the mouse moves, and mx and my are the new window
pixel coordinates, origin at window upper left. */
}
//<<<<<<<<<<<<<<<<<<<<<<<< main >>>>>>>>>>>>>>>>>>>>>>
void main(int argc, char** argv)
{
	glutInit(&argc, argv);          // initialize the toolkit
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB ); // set display mode
	glutInitWindowSize(640,480);     // set window size
	glutInitWindowPosition(100, 150); // set window position on screen
	glutCreateWindow("Lissajous, Ver. 1, Sept 1, 2003, Rudy Rucker"); // open the screen window
	glutDisplayFunc(myDisplay);     // register redraw function
	glutKeyboardFunc(myKeyboard);	// register keyboard function.
	glutMouseFunc(myMouse);			// register mouse click function.
	glutMotionFunc(myMovedMouse);   // register mouse move function.
	glutIdleFunc(myAnimate);		// register idle function to call.
	myInit();                   
	glutMainLoop(); 		     // go into a perpetual loop
}
